#!/usr/bin/env python3
#
# See also:
# https://qpid.apache.org/releases/qpid-proton-0.37.0/
# https://qpid.apache.org/releases/qpid-proton-0.37.0/proton/python/docs/proton.html#proton.Endpoint
# https://qpid.apache.org/releases/qpid-proton-0.37.0/proton/python/docs/proton.html#proton.SASL
# https://qpid.apache.org/releases/qpid-proton-0.37.0/proton/python/docs/proton.html#proton.SSLDomain
# https://qpid.apache.org/releases/qpid-proton-0.37.0/proton/python/docs/proton.handlers.html#proton.handlers.MessagingHandler
# https://qpid.apache.org/releases/qpid-proton-0.37.0/proton/python/docs/proton.reactor.html#proton.reactor.Container
# https://github.com/apache/qpid-proton/blob/0.37.0/python/proton
# https://www.cyrusimap.org/sasl/
#

import collections
import optparse
import uuid

from proton import Endpoint
from proton import SSLDomain
from proton.handlers import MessagingHandler
from proton.reactor import Container


class Queue(object):
    def __init__(self, dynamic=False):
        self.dynamic = dynamic
        self.queue = collections.deque()
        self.consumers = []

    def subscribe(self, consumer):
        self.consumers.append(consumer)

    def unsubscribe(self, consumer):
        """
        :return: True if the queue is to be deleted
        """
        if consumer in self.consumers:
            self.consumers.remove(consumer)
        return len(self.consumers) == 0 and (self.dynamic or len(self.queue) == 0)

    def publish(self, message):
        self.queue.append(message)
        self.dispatch()

    def dispatch(self, consumer=None):
        if consumer:
            c = [consumer]
        else:
            c = self.consumers
        while self._deliver_to(c):
            pass

    def _deliver_to(self, consumers):
        try:
            result = False
            for c in consumers:
                if c.credit:
                    c.send(self.queue.popleft())
                    result = True
            return result
        except IndexError:  # no more messages
            return False


class Broker(MessagingHandler):
    def __init__(self, debug, url, truststore, sslcert, sslkey, sslpass, sasl_config_path, sasl_config_name):
        super(Broker, self).__init__()
        self.debug = debug
        self.url = url
        # SSL trust store
        self.truststore = truststore
        # SSL certificate
        self.sslcert = sslcert
        self.sslkey = sslkey
        self.sslpass = sslpass
        # SASL configuration
        self.sasl_config_path = sasl_config_path
        self.sasl_config_name = sasl_config_name
        # queues
        self.queues = {}

    def on_start(self, event):
        if self.debug >= 1:
            print('on_start:  {}'.format(event))
        ssl_domain = SSLDomain(SSLDomain.MODE_SERVER)
        # SSL trust store (e.g. PEM file containing trusted CA certificate(s))
        if self.truststore:
            ssl_domain.set_trusted_ca_db(self.truststore)

        # server certificate
        if self.sslkey:
            ssl_domain.set_credentials(self.sslcert, self.sslkey, self.sslpass)

        self.acceptor = event.container.listen(url=self.url, ssl_domain=ssl_domain)

    def _queue(self, address):
        if address not in self.queues:
            self.queues[address] = Queue()
        return self.queues[address]

    def on_connection_init(self, event):
        if self.debug >= 1:
            print('on_connection_init:  {}'.format(event))
        # set SASL configuration file
        sasl = event.connection.transport.sasl()
        sasl.config_path(self.sasl_config_path)
        sasl.config_name(self.sasl_config_name)

    def on_link_opening(self, event):
        if self.debug >= 1:
            print('on_link_opening:  {}'.format(event))
        if event.link.is_sender:
            if event.link.remote_source.dynamic:
                address = str(uuid.uuid4())
                event.link.source.address = address
                q = Queue(True)
                self.queues[address] = q
                q.subscribe(event.link)
            elif event.link.remote_source.address:
                event.link.source.address = event.link.remote_source.address
                self._queue(event.link.source.address).subscribe(event.link)
        elif event.link.remote_target.address:
            event.link.target.address = event.link.remote_target.address

    def _unsubscribe(self, link):
        if link.source.address in self.queues and self.queues[link.source.address].unsubscribe(link):
            del self.queues[link.source.address]

    def on_link_closing(self, event):
        if self.debug >= 1:
            print('on_link_closing:  {}'.format(event))
        if event.link.is_sender:
            self._unsubscribe(event.link)

    def on_connection_closing(self, event):
        if self.debug >= 1:
            print('on_connection_closing:  {}'.format(event))
        self.remove_stale_consumers(event.connection)

    def on_disconnected(self, event):
        if self.debug >= 1:
            print('on_disconnected:  {}'.format(event))
        self.remove_stale_consumers(event.connection)

    def remove_stale_consumers(self, connection):
        link = connection.link_head(Endpoint.REMOTE_ACTIVE)
        while link:
            if link.is_sender:
                self._unsubscribe(link)
            link = link.next(Endpoint.REMOTE_ACTIVE)

    def on_sendable(self, event):
        print('on_sendable:  dispatching from queue {}'.format(event.link.source.address))
        self._queue(event.link.source.address).dispatch(event.link)

    def on_message(self, event):
        address = event.link.target.address
        if address is None:
            address = event.message.address
        print('on_message:  publishing message to queue {}'.format(address))
        self._queue(address).publish(event.message)

    # catches event for socket and authentication failures
    def on_transport_error(self, event):
        if self.debug >= 1:
            print('on_transport_error:  {}'.format(event))

        print("Transport error:", event.transport.condition)

    def on_unhandled(self, method, *args):
        if self.debug >= 2:
            print('unhandled event:  {} -- {}'.format(method, args))


def main():
    parser = optparse.OptionParser(usage="usage: %prog [options]")
    parser.add_option("-a", "--address", default="localhost:5671",
                      help="address router listens on (default %default)")
    parser.add_option("-d", "--debug", type=int, default=0,
                      help="debug output level (default %default)")
    parser.add_option("-s", "--truststore", default=None,
                      help="SSL trust store (e.g. cacert.pem)")
    parser.add_option("-c", "--sslcert", default=None,
                      help="SSL certificate / public key (e.g. server-cert.pem)")
    parser.add_option("-k", "--sslkey", default=None,
                      help="SSL private key (e.g. server-key.pem)")
    parser.add_option("-y", "--sslpass", default=None,
                      help="password for client-side SSL private key (overrides ECS_AMQP_SSLPASS env var)")
    parser.add_option("-l", "--sasl_config_path", default=None,
                      help="location of SASL configuration file")
    parser.add_option("-n", "--sasl_config_name", default=None,
                      help="SASL configuration file basename (will have \".conf\" appended)")
    opts, args = parser.parse_args()

    try:
        Container(Broker(opts.debug,
                         opts.address,
                         opts.truststore,
                         opts.sslcert,
                         opts.sslkey,
                         opts.sslpass,
                         opts.sasl_config_path,
                         opts.sasl_config_name)).run()
    except KeyboardInterrupt:
        pass


if __name__ == '__main__':
    main()



#!/usr/bin/env python3
#
# Copyright (C) 2019-2025 National Marrow Donor Program. All rights reserved.
# See also LICENSE file.
#
# Usage example:
# ecs_amqp_send.py --broker amqps://msg01.emdis.net:5672 --vhost default \
#  --address emdis.us.msg --truststore myKeyStore.pem \
#  --sslcert user-cert.pem --sslkey user-key.pem --sslpass PASSWORD \
#  --username emdis-us --password SECRET --inputfile test_msg.txt
#
# See also:
# https://qpid.apache.org/releases/qpid-proton-0.37.0/
# https://qpid.apache.org/releases/qpid-proton-0.37.0/proton/python/docs/proton.html#proton.Message
# https://qpid.apache.org/releases/qpid-proton-0.37.0/proton/python/docs/proton.html#proton.SSLDomain
# https://qpid.apache.org/releases/qpid-proton-0.37.0/proton/python/docs/proton.handlers.html#proton.handlers.MessagingHandler
# https://qpid.apache.org/releases/qpid-proton-0.37.0/proton/python/docs/proton.reactor.html#proton.reactor.Container
#
# Influenced by solace-samples-amqp-qpid-proton-python and cli-proton-python:
# https://github.com/SolaceSamples/solace-samples-amqp-qpid-proton-python
# https://github.com/rh-messaging/cli-proton-python
#

from __future__ import print_function, unicode_literals
import optparse
import os
import sys
from proton import Message
from proton import SSLDomain
from proton.handlers import MessagingHandler
from proton.reactor import Container

# helper function
def get_options():
    parser = optparse.OptionParser(usage="usage: %prog [options]",
                  description="Send message to the supplied address.")
    parser.add_option("-d", "--debug", type=int, default=0,
                  help="debug output level (default %default)")
    parser.add_option("-b", "--broker", default=None,
                  help="amqp message broker host url (e.g. amqps://localhost:5672)")
    parser.add_option("-v", "--vhost", default=None,
                  help="virtual host namespace on broker (e.g. default)")
    parser.add_option("-a", "--address", default=None,
                  help="topic or queue to which message is sent (e.g. test_queue)")
    parser.add_option("-s", "--truststore", default=None,
                  help="SSL trust store (e.g. cacert.pem)")
    parser.add_option("-c", "--sslcert", default=None,
                  help="client-side SSL certificate / public key (e.g. user-cert.pem)")
    parser.add_option("-k", "--sslkey", default=None,
                  help="client-side SSL private key (e.g. user-key.pem)")
    parser.add_option("-y", "--sslpass", default=None,
                  help="password for client-side SSL private key (overrides ECS_AMQP_SSLPASS env var)")
    parser.add_option("-u", "--username", default=None,
                  help="username for SASL authentication")
    parser.add_option("-p", "--password", default=None,
                  help="password for SASL authentication (overrides ECS_AMQP_PASSWORD env var)")
    parser.add_option("-i", "--inputfile", default=None,
                  help="input file containing message to be sent")
    parser.add_option("-P", "--property", action="append", default=None,
                  help="application defined message property and value (e.g. x-emdis-hub-snd=DE)")
    parser.add_option("-E", "--encoding", default='UTF-8',
                  help="message Content-Encoding (default %default)")
    parser.add_option("-T", "--type", default='text/plain',
                  help="message Content-Type (default %default)")
    parser.add_option("-S", "--subject", default=None,
                  help="message Subject (e.g. EMDIS)")
    opts, args = parser.parse_args()
    return opts



"""
Proton event handler class
Creates an amqp connection and a sender to publish messages.
"""
class Send(MessagingHandler):
    def __init__(self, debug, url, vhost, address, truststore, sslcert, sslkey, sslpass, username, password, inputfile, properties, content_encoding, content_type, message_subject):
        super(Send, self).__init__()

        # exit status
        self.exit_status = 1

        # debug output level
        self.debug = debug

        # amqp broker host url
        self.url = url

        # target amqp node address
        self.address = address

        # broker virtual host name
        self.virtual_host = vhost

        # SSL trust store
        self.truststore = truststore

        # client-side SSL certificate
        self.sslcert = sslcert
        self.sslkey = sslkey
        self.sslpass = sslpass

        # authentication credentials
        self.username = username
        self.password = password

        # input file
        self.inputfile = inputfile
        if self.inputfile == '-':
            self.inputfile = sys.stdin.fileno()

        # application defined message properties
        self.application_defined_message_properties = None
        if properties:
            self.application_defined_message_properties = dict([prop.split('=', maxsplit=1) for prop in properties])

        # standard message properties
        self.content_encoding = content_encoding
        self.content_type = content_type
        self.message_subject = message_subject

        # done sending yet?
        self.done_sending = False

    def on_start(self, event):
        if self.debug >= 1:
            print('on_start:  {}'.format(event))

        ssl_domain = SSLDomain(SSLDomain.MODE_CLIENT)
        # SSL trust store (e.g. PEM file containing trusted CA certificate(s))
        if self.truststore:
            ssl_domain.set_trusted_ca_db(self.truststore)
        if True:
            # use trust store to verify peer's (e.g. broker's) SSL certificate
            ssl_domain.set_peer_authentication(SSLDomain.VERIFY_PEER)
        else:
            # verify hostname on peer's (e.g. broker's) SSL certificate
            ssl_domain.set_peer_authentication(SSLDomain.VERIFY_PEER_NAME)

        # client-side certificate
        if self.sslkey:
            ssl_domain.set_credentials(self.sslcert, self.sslkey, self.sslpass)

        if self.username:
            # basic username and password authentication
            event.container.connect(url=self.url,
                                    ssl_domain=ssl_domain,
                                    user=self.username,
                                    password = self.password,
                                    allow_insecure_mechs=False,
                                    sasl_enabled=True,
                                    allowed_mechs="PLAIN",
                                    virtual_host=self.virtual_host)
        else:
            # anonymous authentication
            event.container.connect(url=self.url,
                                    ssl_domain=ssl_domain,
                                    allow_insecure_mechs=False,
                                    sasl_enabled=True,
                                    allowed_mechs="ANONYMOUS",
                                    virtual_host=self.virtual_host)

    def on_connection_opened(self, event):
        if self.debug >= 1:
            print('on_connection_opened:  {}'.format(event))

        # attaches sender link to transmit messages
        event.container.create_sender(event.connection, target=self.address)

    def on_sendable(self, event):
        if self.debug >= 1:
            print('on_sendable:  {}'.format(event))

        if event.sender.credit and not self.done_sending:
            # creates message to send
            msg = None
            with open(self.inputfile, "r") as fd:
                msg = Message(body=fd.read(),
                              subject=self.message_subject,
                              content_type=self.content_type,
                              content_encoding=self.content_encoding,
                              properties=self.application_defined_message_properties,
                              durable=True)
            self.done_sending = True
            if msg:
                # send message
                event.sender.send(msg)
            else:
                self.exit_status = 2
                event.connection.close()

    def on_accepted(self, event):
        if self.debug >= 1:
            print('on_accepted:  {}'.format(event))

        print("Message accepted by broker", self.url)
        self.exit_status = 0
        event.connection.close()

    def on_rejected(self, event):
        if self.debug >= 1:
            print('on_rejected:  {}'.format(event))

        print("Message rejected by broker", self.url)
        print("Delivery tag:", event.delivery.tag)
        self.exit_status = 3
        event.connection.close()

    # catches event for socket and authentication failures
    def on_transport_error(self, event):
        if self.debug >= 1:
            print('on_transport_error:  {}'.format(event))

        print("Transport error:", event.transport.condition)
        self.exit_status = 4
        MessagingHandler.on_transport_error(self, event)

    def on_disconnected(self, event):
        if self.debug >= 1:
            print('on_disconnected:  {}'.format(event))

        if event.transport and event.transport.condition :
            print('Disconnected with error:', event.transport.condition)
            self.exit_status = 5
            event.connection.close()

    def on_unhandled(self, method, *args):
        if self.debug >= 2:
            print('unhandled event:  {} -- {}'.format(method, args))


# get application options
opts = get_options()
# sslpass and password can be set via environment variables,
# but those settings can be overridden via command line options
sslpass = os.environ.get('ECS_AMQP_SSLPASS')
if opts.sslpass:
    sslpass = opts.sslpass
password = os.environ.get('ECS_AMQP_PASSWORD')
if opts.password:
    password = opts.password

"""
The amqp address can be a topic or a queue.
Do not use a prefix or use 'queue://' in the amqp address for 
the amqp sender link target address.
"""

try:
    send = Send(opts.debug,
                opts.broker,
                opts.vhost,
                opts.address,
                opts.truststore,
                opts.sslcert,
                opts.sslkey,
                sslpass,
                opts.username,
                password,
                opts.inputfile,
                opts.property,
                opts.encoding,
                opts.type,
                opts.subject)
    container = Container(send)
    # start proton event reactor
    container.run()
    # check exit status
    if send.exit_status != 0:
        print("exit status", send.exit_status, file=sys.stderr)
    sys.exit(send.exit_status)
except KeyboardInterrupt: pass


#!/usr/bin/env python3
#
# Copyright (C) 2019-2025 National Marrow Donor Program. All rights reserved.
# See also LICENSE file.
#
# Usage example:
# ecs_amqp_recv.py --broker amqps://msg01.emdis.net:5672 --vhost default \
#  --address emdis.us.msg --truststore myKeyStore.pem \
#  --sslcert user-cert.pem --sslkey user-key.pem --sslpass PASSWORD \
#  --username emdis-us --password SECRET --outputdir messages
#
# See also:
# https://qpid.apache.org/releases/qpid-proton-0.37.0/
# https://qpid.apache.org/releases/qpid-proton-0.37.0/proton/python/docs/proton.html#proton.SSLDomain
# https://qpid.apache.org/releases/qpid-proton-0.37.0/proton/python/docs/proton.handlers.html#proton.handlers.MessagingHandler
# https://qpid.apache.org/releases/qpid-proton-0.37.0/proton/python/docs/proton.reactor.html#proton.reactor.Container
#
# Influenced by solace-samples-amqp-qpid-proton-python and cli-proton-python:
# https://github.com/SolaceSamples/solace-samples-amqp-qpid-proton-python
# https://github.com/rh-messaging/cli-proton-python
#

from __future__ import print_function
import optparse
import os
import shutil
import sys
import tempfile
import time
from proton import SSLDomain
from proton.handlers import MessagingHandler
from proton.reactor import Container

# helper function

def get_options():
    parser = optparse.OptionParser(usage="usage: %prog [options]")
    parser.add_option("-d", "--debug", type=int, default=0,
                  help="debug output level (default %default)")
    parser.add_option("-b", "--broker", default=None,
                  help="amqp message broker host url (e.g. amqps://localhost:5672)")
    parser.add_option("-v", "--vhost", default=None,
                  help="virtual host namespace on broker (e.g. default)")
    parser.add_option("-a", "--address", default=None,
                  help="topic or queue from which messages are received (e.g. test_queue)")
    parser.add_option("-s", "--truststore", default=None,
                  help="SSL trust store (e.g. cacert.pem)")
    parser.add_option("-c", "--sslcert", default=None,
                  help="client-side SSL certificate / public key (e.g. user-cert.pem)")
    parser.add_option("-k", "--sslkey", default=None,
                  help="client-side SSL private key (e.g. user-key.pem)")
    parser.add_option("-y", "--sslpass", default=None,
                  help="password for client-side SSL private key (overrides ECS_AMQP_SSLPASS env var)")
    parser.add_option("-u", "--username", default=None,
                  help="username for SASL authentication")
    parser.add_option("-p", "--password", default=None,
                  help="password for SASL authentication (overrides ECS_AMQP_PASSWORD env var)")
    parser.add_option("-t", "--timeout", type=int, default=5,
                  help="inactivity timeout threshold, in seconds (default %default)")
    parser.add_option("-o", "--outputdir", action="append", default=None,
                  help="file system directory for output files")
    parser.add_option("-x", "--suffix", default=".amqp.msg",
                  help="filename suffix for output files (default %default)")

    opts, args = parser.parse_args()

    return opts

"""
Proton event handler class
Creates an amqp connection using ANONYMOUS or PLAIN authentication.
Then attaches a receiver link to consume messages from the broker.
"""
class Recv(MessagingHandler):
    def __init__(self, debug, url, vhost, address, truststore, sslcert, sslkey, sslpass, username, password, timeout, outputdir, suffix):
        super(Recv, self).__init__()

        self.connection = None
        self.receiver = None

        # exit status
        self.exit_status = 1

        # debug output level
        self.debug = debug

        # amqp broker host url
        self.url = url

        # amqp node address
        self.address = address

        # broker virtual host name
        self.virtual_host = vhost

        # SSL trust store
        self.truststore = truststore

        # client-side SSL certificate
        self.sslcert = sslcert
        self.sslkey = sslkey
        self.sslpass = sslpass

        # authentication credentials
        self.username = username
        self.password = password

        # inactivity timeout
        self.inactivity_timestamp = 0
        self.inactivity_threshold = timeout

        # output dir
        self.outputdir = outputdir
        self.hextrans = str.maketrans('0123456789abcdef', 'BCDFGHJKLMNPQRST')
        self.old_prefix_ts = ''
        self.msg_seqnum = 0

        # suffix
        self.suffix = suffix

    def on_start(self, event):
        if self.debug >= 1:
            print('on_start:  {}'.format(event))

        ssl_domain = SSLDomain(SSLDomain.MODE_CLIENT)
        # SSL trust store (e.g. PEM file containing trusted CA certificate(s))
        if self.truststore:
            ssl_domain.set_trusted_ca_db(self.truststore)
        if True:
            # use trust store to verify peer's (e.g. broker's) SSL certificate
            ssl_domain.set_peer_authentication(SSLDomain.VERIFY_PEER)
        else:
            # verify hostname on peer's (e.g. broker's) SSL certificate
            ssl_domain.set_peer_authentication(SSLDomain.VERIFY_PEER_NAME)

        # client-side certificate
        if self.sslkey:
            ssl_domain.set_credentials(self.sslcert, self.sslkey, self.sslpass)

        if self.username:
            # username and password authentication
            self.connection = event.container.connect(url=self.url,
                                                      ssl_domain=ssl_domain,
                                                      user=self.username,
                                                      password = self.password,
                                                      allow_insecure_mechs=False,
                                                      sasl_enabled=True,
                                                      allowed_mechs="PLAIN",
                                                      virtual_host=self.virtual_host)
        else:
            # anonymous authentication
            self.connection = event.container.connect(url=self.url,
                                                      ssl_domain=ssl_domain,
                                                      allow_insecure_mechs=False,
                                                      sasl_enabled=True,
                                                      allowed_mechs="ANONYMOUS",
                                                      virtual_host=self.virtual_host)

    def on_connection_opened(self, event):
        if self.debug >= 1:
            print('on_connection_opened:  {}'.format(event))

        # create receiver link to consume messages
        self.receiver = event.container.create_receiver(event.connection, source=self.address)
        self.exit_status = 0

    def on_connection_closed(self, event):
        if self.debug >= 1:
            print('on_connection_closed:  {}'.format(event))

    def on_reactor_quiesced(self, event):
        if self.debug >= 1:
            print('on_reactor_quiesced [{},{}]:  {}'.format(self.inactivity_timestamp, time.time(), event))

        # TODO: improve robustness of link/session/connection/container teardown (instead of relying solely on inactivity timeout)
        if self.inactivity_timestamp == 0:
            self.inactivity_timestamp = time.time()
        inactivity_seconds = time.time() - self.inactivity_timestamp
        if inactivity_seconds > 0.1 and inactivity_seconds >= self.inactivity_threshold:
            if self.connection:
                self.connection.close()
            else:
                event.container.stop()

    def on_message(self, event):
        if self.debug >= 1:
            print('on_message:  {}'.format(event))

        filename_prefix = int(time.time() * 16).to_bytes(8, byteorder='big').hex().translate(self.hextrans)[8:]
        if self.old_prefix_ts != filename_prefix:
            self.old_prefix_ts = filename_prefix
            # reset msg_seqnum when prefix_ts changes
            self.msg_seqnum = 0
        self.msg_seqnum += 1
        filename_prefix += '%03i.' % self.msg_seqnum

        created_filename = None
        with tempfile.NamedTemporaryFile(dir=self.outputdir[0], mode='wt', prefix=filename_prefix, suffix=self.suffix, delete=False) as fd:
            if event.message.annotations:
                for k, v in event.message.annotations.items():
                    print("{}: {}".format(k,v), file=fd)
            if event.message.instructions:
                for k, v in event.message.instructions.items():
                    print("{}: {}".format(k,v), file=fd)
            if event.message.properties:
                for k, v in event.message.properties.items():
                    print("{}: {}".format(k,v), file=fd)
            if event.message.content_encoding and event.message.content_encoding != 'None':
                print("Content-Encoding: {}".format(event.message.content_encoding), file=fd)
            if event.message.content_type and event.message.content_type != 'None':
                print("Content-Type: {}".format(event.message.content_type), file=fd)
            if event.message.subject:
                print("Subject: {}".format(event.message.subject), file=fd)
            else:
                print("Subject: ", file=fd)
            print("", file=fd)
            print(event.message.body, file=fd)
            fd.flush()
            created_filename = fd.name
        print("Message received and stored in file", created_filename)

        # copy to additional output directories, if specified
        for otherdir in self.outputdir[1:]:
            with open(created_filename, mode='rt') as fd1:
                copied_filename = None
                with tempfile.NamedTemporaryFile(dir=otherdir, mode='wt', prefix=filename_prefix, suffix='.amqp.msg', delete=False) as fd2:
                    shutil.copyfileobj(fd1, fd2)
                    copied_filename = fd2.name
                print("Copied to file", copied_filename)

        self.inactivity_timestamp = 0

    # the on_transport_error event catches socket and authentication failures
    def on_transport_error(self, event):
        if self.debug >= 1:
            print('on_transport_error:  {}'.format(event))

        print("Transport error:", event.transport.condition)
        MessagingHandler.on_transport_error(self, event)

    def on_disconnected(self, event):
        if self.debug >= 1:
            print('on_disconnected:  {}'.format(event))

        print("Disconnected")

    def on_unhandled(self, method, *args):
        if self.debug >= 2:
            print('unhandled event:  {} -- {}'.format(method, args))


# parse arguments and get options
opts = get_options()
# sslpass and password can be set via environment variables,
# but those settings can be overridden via command line options
sslpass = os.environ.get('ECS_AMQP_SSLPASS')
if opts.sslpass:
    sslpass = opts.sslpass
password = os.environ.get('ECS_AMQP_PASSWORD')
if opts.password:
    password = opts.password

"""
The amqp address can be a topic or a queue.
Do not use a prefix or use 'queue://' in the amqp address for
the amqp receiver source.
"""

try:
    recv = Recv(opts.debug,
                opts.broker,
                opts.vhost,
                opts.address,
                opts.truststore,
                opts.sslcert,
                opts.sslkey,
                sslpass,
                opts.username,
                password,
                opts.timeout,
                opts.outputdir,
                opts.suffix)
    container = Container(recv)
    # start proton event reactor
    container.run()
    # check exit status
    if recv.exit_status != 0:
        print("exit status", recv.exit_status, file=sys.stderr)
    sys.exit(recv.exit_status)
except KeyboardInterrupt: pass